<?php

namespace Litecms\Rank\Http\Controllers;

use App\Http\Controllers\ResourceController as BaseController;
use Form;
use Litecms\Rank\Http\Requests\RankRequest;
use Litecms\Rank\Interfaces\RankRepositoryInterface;
use Litecms\Rank\Models\Rank;

/**
 * Resource controller class for rank.
 */
class RankResourceController extends BaseController
{

    /**
     * Initialize rank resource controller.
     *
     * @param type RankRepositoryInterface $rank
     *
     * @return null
     */
    public function __construct(RankRepositoryInterface $rank)
    {
        parent::__construct();
        $this->repository = $rank;
        $this->repository
            ->pushCriteria(\Litepie\Repository\Criteria\RequestCriteria::class)
            ->pushCriteria(\Litecms\Rank\Repositories\Criteria\RankResourceCriteria::class);
    }

    /**
     * Display a list of rank.
     *
     * @return Response
     */
    public function index(RankRequest $request)
    {
        $view = $this->response->theme->listView();

        if ($this->response->typeIs('json')) {
            $function = camel_case('get-' . $view);
            return $this->repository
                ->setPresenter(\Litecms\Rank\Repositories\Presenter\RankPresenter::class)
                ->$function();
        }

        $ranks = $this->repository->paginate();

        return $this->response->title(trans('rank::rank.names'))
            ->view('rank::rank.index', true)
            ->data(compact('ranks'))
            ->output();
    }

    /**
     * Display rank.
     *
     * @param Request $request
     * @param Model   $rank
     *
     * @return Response
     */
    public function show(RankRequest $request, Rank $rank)
    {

        if ($rank->exists) {
            $view = 'rank::rank.show';
        } else {
            $view = 'rank::rank.new';
        }

        return $this->response->title(trans('app.view') . ' ' . trans('rank::rank.name'))
            ->data(compact('rank'))
            ->view($view, true)
            ->output();
    }

    /**
     * Show the form for creating a new rank.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function create(RankRequest $request)
    {

        $rank = $this->repository->newInstance([]);
        return $this->response->title(trans('app.new') . ' ' . trans('rank::rank.name')) 
            ->view('rank::rank.create', true) 
            ->data(compact('rank'))
            ->output();
    }

    /**
     * Create new rank.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(RankRequest $request)
    {
        try {
            $attributes              = $request->all();
            $attributes['user_id']   = user_id();
            $attributes['user_type'] = user_type();
            $rank                 = $this->repository->create($attributes);

            return $this->response->message(trans('messages.success.created', ['Module' => trans('rank::rank.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('rank/rank/' . $rank->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('/rank/rank'))
                ->redirect();
        }

    }

    /**
     * Show rank for editing.
     *
     * @param Request $request
     * @param Model   $rank
     *
     * @return Response
     */
    public function edit(RankRequest $request, Rank $rank)
    {
        return $this->response->title(trans('app.edit') . ' ' . trans('rank::rank.name'))
            ->view('rank::rank.edit', true)
            ->data(compact('rank'))
            ->output();
    }

    /**
     * Update the rank.
     *
     * @param Request $request
     * @param Model   $rank
     *
     * @return Response
     */
    public function update(RankRequest $request, Rank $rank)
    {
        try {
            $attributes = $request->all();

            $rank->update($attributes);
            return $this->response->message(trans('messages.success.updated', ['Module' => trans('rank::rank.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('rank/rank/' . $rank->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('rank/rank/' . $rank->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove the rank.
     *
     * @param Model   $rank
     *
     * @return Response
     */
    public function destroy(RankRequest $request, Rank $rank)
    {
        try {

            $rank->delete();
            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('rank::rank.name')]))
                ->code(202)
                ->status('success')
                ->url(guard_url('rank/rank/0'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('rank/rank/' . $rank->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove multiple rank.
     *
     * @param Model   $rank
     *
     * @return Response
     */
    public function delete(RankRequest $request, $type)
    {
        try {
            $ids = hashids_decode($request->input('ids'));

            if ($type == 'purge') {
                $this->repository->purge($ids);
            } else {
                $this->repository->delete($ids);
            }

            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('rank::rank.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('rank/rank'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/rank/rank'))
                ->redirect();
        }

    }

    /**
     * Restore deleted ranks.
     *
     * @param Model   $rank
     *
     * @return Response
     */
    public function restore(RankRequest $request)
    {
        try {
            $ids = hashids_decode($request->input('ids'));
            $this->repository->restore($ids);

            return $this->response->message(trans('messages.success.restore', ['Module' => trans('rank::rank.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('/rank/rank'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/rank/rank/'))
                ->redirect();
        }

    }

}
